@@ -17,6 +17,7 @@
 extern crate clap;
 #[macro_use]
 extern crate lazy_static;
+use crate::task::Runner;
 use crate::chompfile::ChompTaskMaybeTemplated;
 use crate::chompfile::Chompfile;
 use crate::extensions::init_js_platform;
@@ -24,14 +25,17 @@ use crate::extensions::ExtensionEnvironment;
 use crate::task::expand_template_tasks;
 use anyhow::{anyhow, Result};
 use clap::{App, Arg};
+use std::collections::BTreeMap;
 use std::collections::HashMap;
 use std::collections::HashSet;
-use tokio::fs;
+use std::fs;
 extern crate num_cpus;
 use hyper::Uri;
 use std::env;
 use std::fs::canonicalize;
+use crate::engines::replace_env_vars_static;
 
+mod ansi_windows;
 mod chompfile;
 mod engines;
 mod extensions;
@@ -41,7 +45,7 @@ mod task;
 
 use std::path::PathBuf;
 
-const CHOMP_CORE: &str = "https://ga.jspm.io/npm:@chompbuild/extensions@0.1.8/";
+const CHOMP_CORE: &str = "https://ga.jspm.io/npm:@chompbuild/extensions@0.1.12/";
 
 const CHOMP_INIT: &str = r#"version = 0.1
 
@@ -174,6 +178,14 @@ async fn main() -> Result<()> {
         )
         .get_matches();
 
+    #[cfg(target_os = "windows")]
+    match ansi_windows::enable_ansi_support() {
+        Ok(()) => {},
+        Err(_) => {
+            // TODO: handling disabling of ansi codes
+        }
+    };
+
     let mut targets: Vec<String> = Vec::new();
     match matches.values_of("target") {
         Some(target) => {
@@ -187,7 +199,7 @@ async fn main() -> Result<()> {
     let cfg_file = PathBuf::from(matches.value_of("config").unwrap_or_default());
 
     let mut created = false;
-    let chompfile_source = match fs::read_to_string(&cfg_file).await {
+    let chompfile_source = match fs::read_to_string(&cfg_file) {
         Ok(source) => source,
         Err(_) => {
             if matches.is_present("init") {
@@ -237,8 +249,11 @@ async fn main() -> Result<()> {
     assert!(env::set_current_dir(&cwd).is_ok());
 
     if matches.is_present("clear_cache") {
-        println!("Clearing URL extension cache...");
         http_client::clear_cache().await?;
+        println!("\x1b[1;32m√\x1b[0m Cleared remote URL extension cache.");
+        if targets.len() == 0 {
+            return Ok(());
+        }
     }
 
     init_js_platform();
@@ -248,14 +263,21 @@ async fn main() -> Result<()> {
         None => num_cpus::get(),
     };
 
-    let mut global_env = HashMap::new();
+    let mut global_env = BTreeMap::new();
     for (key, value) in env::vars() {
         global_env.insert(key.to_uppercase(), value);
     }
+    for (key, value) in &chompfile.env {
+        global_env.insert(key.to_uppercase(), replace_env_vars_static(value, &global_env));
+    }
     if matches.is_present("eject_templates") {
         global_env.insert("CHOMP_EJECT".to_string(), "1".to_string());
     }
     global_env.insert("CHOMP_POOL_SIZE".to_string(), pool_size.to_string());
+    // extend global env with the chompfile env as well
+    for (key, value) in &chompfile.env_default {
+        global_env.insert(key.to_uppercase(), replace_env_vars_static(value, &global_env));
+    }
 
     let mut extension_env = ExtensionEnvironment::new(&global_env);
 
@@ -304,7 +326,7 @@ async fn main() -> Result<()> {
                     extension_set.insert(canonical_str.to_string());
                     (
                         extension_set.get(&canonical_str).unwrap(),
-                        Some(fs::read_to_string(&ext).await?),
+                        Some(fs::read_to_string(&ext)?),
                     )
                 } else {
                     (extension_set.get(&canonical_str).unwrap(), None)
@@ -364,12 +386,36 @@ async fn main() -> Result<()> {
         || matches.is_present("import_scripts")
     {
         if matches.is_present("eject_templates") {
-            let (has_templates, template_tasks) =
+            let (mut has_templates, mut template_tasks) =
                 expand_template_tasks(&chompfile, &mut extension_env)?;
-            chompfile.task = template_tasks;
+            chompfile.task = Vec::new();
+            for task in extension_env.get_tasks().drain(..) {
+                has_templates = true;
+                chompfile.task.push(ChompTaskMaybeTemplated {
+                    target: task.target,
+                    targets: task.targets,
+                    args: task.args,
+                    cwd: task.cwd,
+                    dep: task.dep,
+                    deps: task.deps,
+                    display: task.display,
+                    engine: task.engine,
+                    env: task.env.unwrap_or_default(),
+                    env_default: task.env_default.unwrap_or_default(),
+                    env_replace: task.env_replace,
+                    invalidation: task.invalidation,
+                    run: task.run,
+                    name: task.name,
+                    serial: task.serial,
+                    stdio: task.stdio,
+                    template: task.template,
+                    template_options: task.template_options
+                });
+            }
+            chompfile.task.append(&mut template_tasks);
             if !has_templates {
                 return Err(anyhow!(
-                    "\x1b[1m{}\x1b[0m has no template usage to eject",
+                    "\x1b[1m{}\x1b[0m has no templates to eject",
                     cfg_file.to_str().unwrap()
                 ));
             }
@@ -395,11 +441,10 @@ async fn main() -> Result<()> {
                     }
                 }
             }
-            return Ok(());
         } else {
             let mut script_tasks = 0;
             if matches.is_present("import_scripts") {
-                let pjson_source = match fs::read_to_string("package.json").await {
+                let pjson_source = match fs::read_to_string("package.json") {
                     Ok(source) => source,
                     Err(_) => {
                         return Err(anyhow!(
@@ -440,7 +485,7 @@ async fn main() -> Result<()> {
                     _ => return Err(anyhow!("Unexpected \"scripts\" type in package.json.")),
                 };
             }
-            fs::write(&cfg_file, toml::to_string_pretty(&chompfile)?).await?;
+            fs::write(&cfg_file, toml::to_string_pretty(&chompfile)?)?;
             if matches.is_present("eject_templates") {
                 println!(
                     "\x1b[1;32m√\x1b[0m \x1b[1m{}\x1b[0m template tasks ejected.",
@@ -469,26 +514,22 @@ async fn main() -> Result<()> {
                     if created { "created" } else { "updated" }
                 );
             }
-            if matches.is_present("eject_templates") || targets.len() == 0 {
-                return Ok(());
-            }
+        }
+        if matches.is_present("eject_templates") || targets.len() == 0 {
+            return Ok(());
         }
     }
 
-    let ok = task::run(
-        &chompfile,
-        &mut extension_env,
-        task::RunOptions {
-            watch: matches.is_present("serve") || matches.is_present("watch"),
-            force: matches.is_present("force"),
-            rerun: matches.is_present("rerun"),
-            args: if args.len() > 0 { Some(args) } else { None },
-            pool_size,
-            targets,
-            cfg_file,
-        },
-    )
-    .await?;
+    let mut runner = Runner::new(&chompfile, &mut extension_env, pool_size, matches.is_present("serve") || matches.is_present("watch"))?;
+    let ok = runner.run(task::RunOptions {
+        watch: matches.is_present("serve") || matches.is_present("watch"),
+        force: matches.is_present("force"),
+        rerun: matches.is_present("rerun"),
+        args: if args.len() > 0 { Some(args) } else { None },
+        pool_size,
+        targets,
+        cfg_file,
+    }).await?;
 
     if !ok {
         eprintln!("Unable to complete all tasks.");
